@@ -45,6 +45,8 @@ struct GnomeIdleMonitorPrivate
 	XSyncCounter counter;
 
 	XSyncAlarm   xalarm_reset;
+
+	GdkDevice   *device;
 };
 
 typedef struct
@@ -57,6 +59,15 @@ typedef struct
 	XSyncAlarm		  xalarm;
 } GnomeIdleMonitorWatch;
 
+enum
+{
+	PROP_0,
+	PROP_DEVICE,
+	PROP_LAST,
+};
+
+static GParamSpec *obj_props[PROP_LAST];
+
 enum
 {
 	BECAME_ACTIVE,
@@ -76,15 +87,7 @@ _xsyncvalue_to_int64 (XSyncValue value)
 		| (guint64) XSyncValueLow32 (value);
 }
 
-static XSyncValue
-_int64_to_xsyncvalue (gint64 value)
-{
-	XSyncValue ret;
-
-	XSyncIntsToValue (&ret, value, ((guint64)value) >> 32);
-
-	return ret;
-}
+#define GINT64_TO_XSYNCVALUE(value, ret) XSyncIntsToValue (ret, value, ((guint64)value) >> 32)
 
 static gboolean
 _find_alarm (gpointer		    key,
@@ -112,7 +115,7 @@ _xsync_alarm_set (GnomeIdleMonitor	*monitor,
 	attr.delta = delta;
 	attr.events = TRUE;
 
-	attr.trigger.wait_value = _int64_to_xsyncvalue (interval);
+	GINT64_TO_XSYNCVALUE (interval, &attr.trigger.wait_value);
 	attr.trigger.test_type = test_type;
 	return XSyncCreateAlarm (monitor->priv->display, flags, &attr);
 }
@@ -175,22 +178,39 @@ xevent_filter (GdkXEvent	*xevent,
 	return GDK_FILTER_CONTINUE;
 }
 
+static char *
+counter_name_for_device (GdkDevice *device)
+{
+	if (device) {
+		gint device_id = gdk_x11_device_get_id (device);
+		if (device_id > 0)
+			return g_strdup_printf ("DEVICEIDLETIME %d", device_id);
+	}
+
+	return g_strdup ("IDLETIME");
+}
+
 static XSyncCounter
-find_idletime_counter (Display *display)
+find_idletime_counter (GnomeIdleMonitor *monitor)
 {
 	int		    i;
 	int		    ncounters;
 	XSyncSystemCounter *counters;
+	XSyncCounter        counter = None;
+	char               *counter_name;
 
-	counters = XSyncListSystemCounters (display, &ncounters);
+	counter_name = counter_name_for_device (monitor->priv->device);
+	counters = XSyncListSystemCounters (monitor->priv->display, &ncounters);
 	for (i = 0; i < ncounters; i++) {
-		if (counters[i].name != NULL && strcmp (counters[i].name, "IDLETIME") == 0) {
-			return counters[i].counter;
+		if (counters[i].name != NULL && strcmp (counters[i].name, counter_name) == 0) {
+			counter = counters[i].counter;
+			break;
 		}
 	}
 	XSyncFreeSystemCounterList (counters);
+	g_free (counter_name);
 
-	return None;
+	return counter;
 }
 
 static guint32
@@ -252,7 +272,7 @@ init_xsync (GnomeIdleMonitor *monitor)
 		return;
 	}
 
-	monitor->priv->counter = find_idletime_counter (monitor->priv->display);
+	monitor->priv->counter = find_idletime_counter (monitor);
 	if (monitor->priv->counter == None) {
 		g_warning ("GnomeIdleMonitor: IDLETIME counter not found");
 		return;
@@ -274,27 +294,56 @@ gnome_idle_monitor_dispose (GObject *object)
 		monitor->priv->watches = NULL;
 	}
 
+	gdk_window_remove_filter (NULL, (GdkFilterFunc)xevent_filter, monitor);
+
+	g_clear_object (&monitor->priv->device);
+
 	G_OBJECT_CLASS (gnome_idle_monitor_parent_class)->dispose (object);
 }
 
-static GObject *
-gnome_idle_monitor_constructor (GType                  type,
-                                guint                  n_construct_properties,
-                                GObjectConstructParam *construct_properties)
+static void
+gnome_idle_monitor_get_property (GObject    *object,
+				 guint       prop_id,
+				 GValue     *value,
+				 GParamSpec *pspec)
 {
-	static GnomeIdleMonitor *global_monitor = NULL;
-
-	if (g_once_init_enter (&global_monitor)) {
-		GnomeIdleMonitor *monitor;
-
-		monitor = GNOME_IDLE_MONITOR (G_OBJECT_CLASS (gnome_idle_monitor_parent_class)->constructor (type,
-													     n_construct_properties,
-													     construct_properties));
-
-		g_once_init_leave (&global_monitor, monitor);
+	GnomeIdleMonitor *monitor = GNOME_IDLE_MONITOR (object);
+	switch (prop_id)
+	{
+	case PROP_DEVICE:
+		g_value_set_object (value, monitor->priv->device);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
 	}
+}
 
-	return g_object_ref (global_monitor);
+static void
+gnome_idle_monitor_set_property (GObject      *object,
+				 guint         prop_id,
+				 const GValue *value,
+				 GParamSpec   *pspec)
+{
+	GnomeIdleMonitor *monitor = GNOME_IDLE_MONITOR (object);
+	switch (prop_id)
+	{
+	case PROP_DEVICE:
+		monitor->priv->device = g_value_dup_object (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+gnome_idle_monitor_constructed (GObject *object)
+{
+	GnomeIdleMonitor *monitor = GNOME_IDLE_MONITOR (object);
+
+	monitor->priv->display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+	init_xsync (monitor);
 }
 
 static void
@@ -303,7 +352,22 @@ gnome_idle_monitor_class_init (GnomeIdleMonitorClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->dispose = gnome_idle_monitor_dispose;
-	object_class->constructor = gnome_idle_monitor_constructor;
+	object_class->constructed = gnome_idle_monitor_constructed;
+	object_class->get_property = gnome_idle_monitor_get_property;
+	object_class->set_property = gnome_idle_monitor_set_property;
+
+	/**
+	 * GnomeIdleMonitor:device:
+	 *
+	 * The device to listen to idletime on.
+	 */
+	obj_props[PROP_DEVICE] =
+		g_param_spec_object ("device",
+				     "Device",
+				     "The device to listen to idletime on",
+				     GDK_TYPE_DEVICE,
+				     G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+	g_object_class_install_property (object_class, PROP_DEVICE, obj_props[PROP_DEVICE]);
 
         signals[BECAME_ACTIVE] =
                 g_signal_new ("became-active",
@@ -332,67 +396,85 @@ gnome_idle_monitor_init (GnomeIdleMonitor *monitor)
 							NULL,
 							NULL,
 							(GDestroyNotify)idle_monitor_watch_free);
-	monitor->priv->counter = None;
-
-	monitor->priv->display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-	init_xsync (monitor);
 }
 
+/**
+ * gnome_idle_monitor_new:
+ *
+ * Returns: a new #GnomeIdleMonitor that tracks the server-global
+ * idletime for all devices. To track device-specific idletime,
+ * use gnome_idle_monitor_new_for_device().
+ */
 GnomeIdleMonitor *
 gnome_idle_monitor_new (void)
 {
 	return GNOME_IDLE_MONITOR (g_object_new (GNOME_TYPE_IDLE_MONITOR, NULL));
 }
 
+/**
+ * gnome_idle_monitor_new_for_device:
+ * @device: A #GdkDevice to get the idle time for.
+ *
+ * Returns: a new #GnomeIdleMonitor that tracks the device-specific
+ * idletime for @device. To track server-global idletime for all
+ * devices, use gnome_idle_monitor_new().
+ */
+GnomeIdleMonitor *
+gnome_idle_monitor_new_for_device (GdkDevice *device)
+{
+	return GNOME_IDLE_MONITOR (g_object_new (GNOME_TYPE_IDLE_MONITOR,
+						 "device", device, NULL));
+}
+
 /**
  * gnome_idle_monitor_add_watch:
  * @monitor: A #GnomeIdleMonitor
- * @interval: The idletime interval, in milliseconds
+ * @interval_msec: The idletime interval, in milliseconds
  * @callback: (allow-none): The callback to call when the user has
  *     accumulated @interval seconds of idle time.
  * @user_data: (allow-none): The user data to pass to the callback
  * @notify: A #GDestroyNotify
- * @watch_id: (out): The watch ID
+ *
+ * Returns: a watch id
  *
  * Adds a watch for a specific idle time. The callback will be called
- * when the user has accumlated @interval seconds of idle time.
- * This function will return an ID in @watch_id that can either be passed
- * to gnome_idle_monitor_remove_watch(), or can be used to tell idle time
+ * when the user has accumulated @interval_msec milliseconds of idle time.
+ * This function will return an ID that can either be passed to
+ * gnome_idle_monitor_remove_watch(), or can be used to tell idle time
  * watches apart if you have more than one.
  *
  * If you need to check for more than one interval of
  * idle time, it may be more convenient to connect to the
- * #GnomeIdleMonitor:::triggered-idle signal, and switch on the
- * watch ID that is passed to the signal.
+ * #GnomeIdleMonitor:::triggered-idle signal, and filter on the
+ * watch ID received in the signal handler.
  *
  * Also note that this function will only care about positive transitions
  * (user's idle time exceeding a certain time). If you want to know about
  * negative transitions, connect to the #GnomeIdleMonitor:::became-active
  * signal.
  */
-void
+guint
 gnome_idle_monitor_add_watch (GnomeIdleMonitor	       *monitor,
-			      guint			interval,
+			      guint			interval_msec,
 			      GnomeIdleMonitorWatchFunc callback,
 			      gpointer			user_data,
-			      GDestroyNotify		notify,
-			      guint                    *watch_id)
+			      GDestroyNotify		notify)
 {
 	GnomeIdleMonitorWatch *watch;
 
-	g_return_if_fail (GNOME_IS_IDLE_MONITOR (monitor));
+	g_return_val_if_fail (GNOME_IS_IDLE_MONITOR (monitor), 0);
 
 	watch = idle_monitor_watch_new ();
 	watch->display = monitor->priv->display;
 	watch->callback = callback;
 	watch->user_data = user_data;
 	watch->notify = notify;
-	watch->xalarm = _xsync_alarm_set (monitor, XSyncPositiveTransition, interval);
+	watch->xalarm = _xsync_alarm_set (monitor, XSyncPositiveTransition, interval_msec);
 
 	g_hash_table_insert (monitor->priv->watches,
 			     GUINT_TO_POINTER (watch->id),
 			     watch);
-	*watch_id = watch->id;
+	return watch->id;
 }
 
 /**
@@ -417,7 +499,7 @@ gnome_idle_monitor_remove_watch (GnomeIdleMonitor *monitor,
  * gnome_idle_monitor_get_idletime:
  * @monitor: A #GnomeIdleMonitor
  *
- * Returns: The user's current idle time, in milliseconds.
+ * Returns: The current idle time, in milliseconds.
  */
 gint64
 gnome_idle_monitor_get_idletime (GnomeIdleMonitor *monitor)
