@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #ifndef __GNOME_XKB_INFO_H__
@@ -74,6 +74,14 @@ gboolean        gnome_xkb_info_get_layout_info_for_language     (GnomeXkbInfo *s
                                                                  const gchar **short_name,
                                                                  const gchar **xkb_layout,
                                                                  const gchar **xkb_variant);
+GList          *gnome_xkb_info_get_all_option_groups            (GnomeXkbInfo *self);
+const gchar    *gnome_xkb_info_description_for_group            (GnomeXkbInfo *self,
+                                                                 const gchar  *group_id);
+GList          *gnome_xkb_info_get_options_for_group            (GnomeXkbInfo *self,
+                                                                 const gchar  *group_id);
+const gchar    *gnome_xkb_info_description_for_option           (GnomeXkbInfo *self,
+                                                                 const gchar  *group_id,
+                                                                 const gchar  *id);
 
 void            gnome_xkb_info_get_var_defs                     (gchar            **rules,
                                                                  XkbRF_VarDefsRec **var_defs);
